import argparse
import os
import csv
import sys
import matplotlib.pyplot as plt
import itertools
# import seaborn as sns
import collections

import pandas as pd


def statistics(csv_path):
    data = pd.read_csv(csv_path)
    # data = data.sort_values(by=['target'])
    print(data)
    var = data.var()
    print(var[0])
    print(var)
    var_list = []
    for n, val in var.iteritems():
        var_list.append([n, val])
    var_list.sort(key=lambda i: i[0])




    data['variance'] = var
    data = data.sort_values(by=['target'])

    fig, ax = plt.subplots(figsize=(40, 5))
    # ax.bar(data['target'], data['variance'])
    print([i[0] for i in var_list], [i[1] for i in var_list])
    ax.bar([i[0] for i in var_list], [i[1] for i in var_list])
    plt.xticks(rotation=30, ha='right')
    plt.show()

    averaged_var = {}

    for name, val in var_list:
        key = name.split('_')[0]
        if key not in averaged_var.keys():
            averaged_var[key] = [val, 1]
        else:
            averaged_var[key] = [averaged_var[key][0] + val, averaged_var[key][1]+1]

    print(averaged_var)
    for key, val in averaged_var.items():
        averaged_var[key] = val[0]/val[1]
    print(averaged_var)
    fig1, ax1 = plt.subplots(figsize=(10, 5))
    ax1.bar(averaged_var.keys(), averaged_var.values())
    plt.xticks(rotation=30, ha='right')
    plt.show()

    # data.to_csv("/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/var_0.0-2.3_2.csv")


def filter_data1(dataframe):
    dataframe = dataframe.sort_values(by=['target'])
    new_frame = pd.DataFrame()
    length = len(dataframe['target'])

    for index, series in dataframe.iterrows():

        correlations = []
        ref_type = series["target"].split("_")[0]
        for a, b in series.iteritems():
            # print(a, b)
            if a == 'target':
                pass
            else:
                res_type = a.split('_')[0]
                if res_type != ref_type:
                    correlations.append(b)
        if len(correlations) < length:
            for i in range(length-len(correlations)):
                correlations.append(None)
        new_frame[series["target"]] = correlations
    variance = new_frame.mean()
    print(variance)

    averaged_var = {}

    for name, val in variance.iteritems():
        key = name.split('_')[0]
        if key not in averaged_var.keys():
            averaged_var[key] = [val, 1]
        else:
            averaged_var[key] = [averaged_var[key][0] + val, averaged_var[key][1] + 1]
    print(averaged_var)
    for key, val in averaged_var.items():
        averaged_var[key] = val[0] / val[1]
    print(averaged_var)

    fig1, ax1 = plt.subplots(figsize=(10, 5))
    ax1.bar(averaged_var.keys(), averaged_var.values())
    plt.xticks(rotation=30, ha='right')
    ax1.set_ylim(0.6, 1)
    plt.show()
    new_frame.to_csv("/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/filtered_3.5-4.0.csv")
    tmp_dict = {}

def filter_data(dataframe):
    dataframe = dataframe.sort_values(by=['target'])
    new_frame = pd.DataFrame()
    length = len(dataframe['target'])

    for index, series in dataframe.iterrows():

        correlations = []
        ref_type = series["target"].split("_")[0]
        for a, b in series.iteritems():
            # print(a, b)
            if a == 'target':
                pass
            else:
                res_type = a.split('_')[0]
                if res_type != ref_type:
                    correlations.append(b)
        if len(correlations) < length:
            for i in range(length - len(correlations)):
                correlations.append(None)
        new_frame[series["target"]] = correlations
    return new_frame


def extract_max_corr(dataframe):
    dataframe = dataframe.sort_values(by=['target'], ascending=False)
    new_frame = pd.DataFrame()
    # length = len(dataframe['target'])

    for index, series in dataframe.iterrows():
        tmp_dict = {}
        ref_type = series["target"].split("_")[0]
        tmp_dict[ref_type] = 1
        for a, b in series.iteritems():
            # print(a, b)
            if a == 'target':
                pass
            else:
                res_type = a.split('_')[0]
                if res_type != ref_type:
                    if res_type not in tmp_dict.keys():
                        tmp_dict[res_type] = round(b, 5)
                    elif tmp_dict[res_type] < b:
                        tmp_dict[res_type] = round(b, 5)

        ordered_dict = collections.OrderedDict(sorted(tmp_dict.items()))

        new_frame[series["target"]] = ordered_dict.values()
    new_frame.insert(loc=0, column='res', value=ordered_dict.keys())
    new_frame = new_frame.set_index('res')
    return new_frame





def comute_mean(frame):
    mean_series = frame.mean()
    return mean_series

def average_over_rotamers(series):
    averaged_val = {}

    for name, val in series.iteritems():
        key = name.split('_')[0]
        if key not in averaged_val.keys():
            averaged_val[key] = [val, 1]
        else:
            averaged_val[key] = [averaged_val[key][0] + val, averaged_val[key][1] + 1]
    # print(averaged_val)
    for key, val in averaged_val.items():
        averaged_val[key] = val[0] / val[1]
    return averaged_val

def series_to_dict(series):
    dic_data = {}

    for name, val in series.iteritems():
            dic_data[name] = val
    return dic_data



def plot_single(data_dict, resolution_range, plot_path=None):
    fig, ax = plt.subplots(figsize=(80, 10))
    ax.bar(data_dict.keys(), data_dict.values())
    plt.xticks(rotation=45, ha='right')
    ax.set_ylim(0.6, 1)
    plt.title(f'Resolution band: {resolution_range}')
    ax.set_xlabel('Motif')
    ax.set_ylabel('Average correlation')
    plt.subplots_adjust(bottom=0.55)
    if plot_path is None:
        plt.show()
    else:
        fig.savefig(plot_path)


def plot_multiple(data_list, plot_path=None, y_lim=(0, 1)):
    # hatches = itertools.cycle(['///', '+++', 'oo', 'XX', 'OO', '.', '--'])

    fig, ax = plt.subplots(figsize=(12, 5))
    width = 0.4
    dd = width*len(data_list)/2 + 1
    x_ticks = [i for i in range(20)]
    x_ticks = [ x * dd for x in x_ticks]
    print(len(x_ticks))
    ax.set_xticks([ x + width * len(data_list)/2 for x in x_ticks])
    colors = ['blue', 'orange', 'teal', 'brown']
    colors = ['#009E73', '#F0E442', '#0072B2', '#D55E00']
    colors = ['#D81B60', '#1E88E5', '#FFC107', '#004D40']
    i = 0
    for data, res_range in data_list:
        print(res_range)
        print(len(data))
        x_ticks = [x+width for x in x_ticks]
        # ax.bar(data.keys(), data.values(), label=f'{res_range}')
        # hatch = next(hatches)
        bar = ax.bar(x_ticks, data.values(), width, label=f'{res_range} $\AA$', color=colors[i])
        print(bar)
        i += 1
        # for b in bar:
        #     b.set_hatch(hatch)
    ax.legend(prop={'size': 13})

    ax.set_xticklabels([i.capitalize() for i in data_list[0][0].keys()], rotation=0)
    ax.set_xlim(-2*width, x_ticks[-1] + 2*width)
    ax.set_ylim(y_lim)
    x = plt.xlabel('Motif type')
    x.set_fontsize(26)
    # ax.set_ylabel('1-correlation')
    y = plt.ylabel('1-correlation')
    y.set_fontsize(26)
    plt.xticks(fontsize=16)
    plt.yticks(fontsize=18)

    if plot_path is None:
        plt.show()
    else:
        plt.subplots_adjust(bottom=0.15)
        fig.savefig(plot_path, format='jpg', dpi=600)


def plot_multiple_non_av(data_list, plot_path=None, y_lim=(0, 1)):
    fig, ax = plt.subplots(figsize=(12, 5))
    width = 0.25
    dd = width*len(data_list)/2 + 1
    x_ticks = [i for i in range(len(data_list[0][0]))]
    x_ticks = [ x * dd for x in x_ticks]
    print(len(x_ticks))
    ax.set_xticks([ x + width * len(data_list)/2 for x in x_ticks])

    for data, res_range in data_list:
        print(res_range)
        print(len(data))
        x_ticks = [x+width for x in x_ticks]
        # ax.bar(data.keys(), data.values(), label=f'{res_range}')
        ax.bar(x_ticks, data.values(), width, label=f'{res_range}')
    ax.legend()

    ax.set_xticklabels(data_list[0][0].keys(), rotation=65)
    ax.set_xlim(0, x_ticks[-1] + 5.5)
    ax.set_ylim(y_lim)
    ax.set_xlabel('Motif type')
    ax.set_ylabel('Average correlation')
    if plot_path is None:
        plt.show()
    else:
        plt.subplots_adjust(bottom=0.15)
        fig.savefig(plot_path)

# def filter_max_

def one_minus_val(val_dict):
    for key, val in val_dict.items():
        val_dict[key] = 1 - val
    return val_dict


def plot_all_average_corr(data_path, out_folder=None, action='mean'):
    files = os.listdir(data_path)
    csv_files = [f for f in files if f.endswith('.csv') and not f.startswith('.')]
    csv_files.sort()
    data_list = []
    tmp = csv_files[:3] + [csv_files[-1]]
    # tmp = csv_files
    for file in tmp:
        print(file)
        res_range = file.split('.csv')[0].split('_')[-1]
        path = os.path.join(data_path, file)
        data = pd.read_csv(path)
        filtered = filter_data(data)
        if action == 'mean':
            series = comute_mean(filtered)
        else:
            return
        rot_aver = average_over_rotamers(series)
        print(rot_aver)
        rot_aver = one_minus_val(rot_aver)

        data_list.append([rot_aver, res_range])

    plot_multiple(data_list, y_lim=(0, 0.3), plot_path=os.path.join(out_folder, 'aver_corr_3+1_one_minus_no_hatch_final1.jpg'))


def plot_all_corr(data_path, out_folder=None, action='mean'):
    files = os.listdir(data_path)
    csv_files = [f for f in files if f.endswith('.csv')]
    csv_files.sort()
    data_list = []

    for file in csv_files:
        res_range = file.split('.csv')[0]
        path = os.path.join(data_path, file)
        data = pd.read_csv(path)
        filtered = filter_data(data)
        if action == 'mean':
            series = comute_mean(filtered)
        else:
            return
        rot_aver = series_to_dict(series)

        data_list.append([rot_aver, res_range])

        plot_single(rot_aver, res_range, plot_path=os.path.join(out_folder,f'{res_range}_all_rot.png'))
        break


def plot_grid_similarity(df, plot_path):
    import numpy as np

    import matplotlib as mpl
    mpl.rcParams['xtick.top'] = True
    mpl.rcParams['xtick.labeltop'] = True

    fig, ax = plt.subplots(figsize=(16, 36))

    from matplotlib import colors
    cmap = colors.ListedColormap(['white', 'blue', 'white'])
    bounds = [0, 0.95, 0.999, 1]
    norm = colors.BoundaryNorm(bounds, cmap.N)

    ax.pcolor(df, cmap=cmap, norm=norm)

    y_labels = [l.split('_') for l in df.index]

    form_labels = []
    for l in y_labels:
        if l[0] in ['ala', 'gly']:
            txt = f'{l[0]}'
        else:
            txt = f'{l[0]} {l[-1]}'
        form_labels.append(txt)

    ax.grid(which='major', axis='both', linestyle='-', color='k', linewidth=0.5)
    plt.yticks(np.arange(0.99, len(df.index), 1), form_labels, va='top', ha='right')
    print(np.arange(0.5, len(df.columns), 1))
    print(df.columns)
    labels = [l + '  ' for l in df.columns]
    plt.xticks(np.arange(0.99, len(df.columns), 1), labels, ha='right')

    plt.tick_params(axis='x', which='major', labelsize=18)
    plt.tick_params(axis='y', which='major', labelsize=18)
    if plot_path is None:
        plt.show()
    else:
        plt.subplots_adjust(bottom=0.15)
        fig.savefig(plot_path)



def generate_all_drid_similarity(data_path, out_folder=None):
    files = os.listdir(data_path)
    csv_files = [f for f in files if f.endswith('.csv') and not f.startswith('.')]
    csv_files.sort()
    data_list = []
    tmp = csv_files
    # tmp = csv_files
    for file in tmp:
        print(file)
        res_range = file.split('.csv')[0].split('_')[-1]
        path = os.path.join(data_path, file)
        data = pd.read_csv(path)
        filtered = extract_max_corr(data)
        df = filtered.transpose()
        # df = df.sort_index(axis=1)


        plot_grid_similarity(df, os.path.join(out_folder,f'{res_range}_sim_grid.png'))



# path = '/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/2.3-2.5.csv'

# data = pd.read_csv(path)
# filter_data(data)

# statistics(path)
data_path = '/Users/andrei/Documents/Project_data/3D-STRUDEL/libs.3/out'
fig_path = '/Users/andrei/Documents/Project_data/3D-STRUDEL/libs.3/pics_2'
if not os.path.exists(fig_path):
    os.makedirs(fig_path)
plot_all_average_corr(data_path, out_folder=fig_path)
## plot_all_corr(data_path, out_folder=fig_path)

# generate_all_drid_similarity(data_path, out_folder=fig_path)



# tmp_path = '/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/libs_similarity/0.0-2.3.csv'
# tmp_path = '/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/libs_similarity/2.8-3.0.csv'
# data = pd.read_csv(tmp_path)

# new_data = extract_max_corr(data)
# new_data.to_csv('/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/libs_similarity/0.0-2.3_max.csv')
# new_data.to_csv('/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/libs_similarity/2.8-3.0_max.csv')


# no_res = new_data.drop('res', 1)
# np_data = new_data.to_numpy()
# from matplotlib import colors
#
# cmap = colors.ListedColormap(['white', 'blue', 'white'])
# bounds = [0,0.95,0.999, 1]
# norm = colors.BoundaryNorm(bounds, cmap.N)
#
# fig, ax = plt.subplots()
# ax.imshow(np_data, cmap=cmap, norm=norm)
import numpy as np
# # draw gridlines
# ax.grid(which='major', axis='both', linestyle='-', color='k', linewidth=0.5)
# ax.set_xticks(np.arange(-.5, 130, 1))
# ax.set_yticks(np.arange(-.5, 20, 1))
#
# plt.show()

# df = new_data.transpose()
# df = df.sort_index(axis=1)
# df.to_csv('/Volumes/data/libs/from_ftp/strudel-libs_ver-2.0_voxel-0.5/libs_similarity/0.0-2.3_max_transp.csv')
#
# print(df.head())
# import matplotlib as mpl
#
# mpl.rcParams['xtick.top'] = True
# mpl.rcParams['xtick.labeltop'] = True
#
# fig, ax = plt.subplots(figsize=(16, 36))
# # plt.rcParams['xtick.top'] = plt.rcParams['xtick.labeltop'] = True
# # plt.pcolor(df)
# from matplotlib import colors
# cmap = colors.ListedColormap(['white', 'blue', 'white'])
# bounds = [0,0.95,0.999, 1]
# norm = colors.BoundaryNorm(bounds, cmap.N)
# # ax.imshow(df, cmap=cmap, norm=norm)
# ax.pcolor(df, cmap=cmap, norm=norm)
# # plt.pcolor(df)
# y_labels = [l.split('_') for l in df.index]
# print(len(y_labels), 'y_lb')
# form_labels = []
# for l in y_labels:
#     if l[0] in ['ala', 'gly']:
#         txt = f'{l[0]}'
#     else:
#         txt = f'{l[0]} {l[-1]}'
#     form_labels.append(txt)
# print(form_labels)
# print(len(form_labels), 'f_lb')
# y_labels = [l[0] + ' ' + l[-1] for l in y_labels]
#
# ax.grid(which='major', axis='both', linestyle='-', color='k', linewidth=0.5)
# plt.yticks(np.arange(0.99, len(df.index), 1), form_labels, va='top', ha='right')
# print(np.arange(0.5, len(df.columns), 1))
# print(df.columns)
# labels = [l + '   ' for l in df.columns]
# plt.xticks(np.arange(0.99, len(df.columns), 1), labels, ha='right')
# # def same(x):
# #     return x
# # secaxx = ax.secondary_xaxis('top')
# plt.tick_params(axis='x', which='major', labelsize=16)
# plt.tick_params(axis='y', which='major', labelsize=14)
# plt.show()
#
# # import seaborn as sns
# #
# # sns.heatmap(new_data, annot=True)


"""
mapMotifValidation.py

This module runs map versus amino acid residues map motif library validation.

Copyright [2013] EMBL - European Bioinformatics Institute
Licensed under the Apache License, Version 2.0 (the
"License"); you may not use this file except in
compliance with the License. You may obtain a copy of
the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied. See the License for the
specific language governing permissions and limitations
under the License.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

__author__ = 'Andrei Istrate'
__email__ = 'andrei@ebi.ac.uk'
__date__ = '2018-05-29'

import os
import sys
import argparse
import subprocess
from shutil import copy
from distutils.spawn import find_executable
from threed_strudel import configure as config
from threed_strudel.utils import functions as func
import threed_strudel.utils.cluster_utils as cu



def get_motif_resolution_ranges(library_path):
    lib_list = []
    folders = os.listdir(library_path)
    for folder in folders:
        if folder.startswith('motifs_'):
            res_range = folder.split('_')[-1].split('-')
            res_range = [float(i) for i in res_range]
            lib_list.append([folder, res_range])
    return lib_list


def main():
    parser = argparse.ArgumentParser(description='Map Validation')
    parser.add_argument("-i", "--json_input", dest="json_inp", required=True,
                        help="List of models maps resolution and out folder in json format. "
                             "e.g.: [[run_flag(0 or 1), map, model, resolution, out_folder]]")
    parser.add_argument("-o", "--out", dest="out", required=True, help="Output directory")
    parser.add_argument("-l", "--lib", dest="lib", required=True, help="Strudel motif libraries path")
    parser.add_argument("-M", "--memory", dest="mem", required=True, help="Maximum memory")
    parser.add_argument("-np", "--num_proc", dest="np", required=True, help="Number of processors")
    parser.add_argument("-v", "--voxel", dest="voxel", required=False, default=0.25, type=float,
                        help="Segments voxel size")
    parser.add_argument("-s", "--keep_segments", dest="segm", action='store_true',
                        help="Keep residues segments after finishing")
    # parser.add_argument("-process_log", "--process_log", dest="process_log", default=None, required=False, help="Log file")
    # parser.add_argument("-v", "--voxel", dest="voxel", required=False, default=0.25, type=float, help="Segments voxel size")
    # parser.add_argument("-r", "--recompute", dest="recompute_scores", action='store_true',
    #                     help="Recalculate correlations")
    # parser.add_argument("-rs", "--recompute_segments", dest="recompute_segments", action='store_true',
    #                     help="Repeat segmenting of the input model and map if found in the output")
    # parser.add_argument("-vc", "--verbose_chimerax", dest="v_c", action='store_true',
    #                     help="Run ChimeraX in verbose mode")
    # parser.add_argument("-wl", "--warning_level", dest="warning_level", default='info',
    #                     help="Log file warning level [info[debug]")


    if not find_executable(config.CHIMERA_PATH):
        print('ChimeraX path not set!\nPlease run strudel_setChimeraX.py to set the ChimeraX path')
        sys.exit()


    print(' '.join(sys.argv))
    args = parser.parse_args()
    segm = args.segm
    if segm:
        segm = '-s'
    else:
        segm = ''
    # if args.log:
    #     log_file = args.log
    # else:
    #     log_file = os.path.join(args.out, 'map_motif_validation.process_log')

    lib_list = get_motif_resolution_ranges(args.lib)

    entry_list = func.read_json(args.json_inp)
    if not os.path.exists(args.out):
        os.makedirs(args.out)
    tmp = os.path.join(args.out, 'tmp')
    if not os.path.exists(tmp):
        os.makedirs(tmp)
    for entry in entry_list:
        if entry[0]:
            resolution = entry[3]
            for lib in lib_list:
                if lib[1][0] < resolution <= lib[1][1]:
                    lib_path = os.path.join(args.lib, lib[0])
                    out_path = os.path.join(args.out, entry[4])
                    if not os.path.exists(out_path):
                        os.makedirs(out_path)
                    # command = f'strudel_mapMotifValidation.py ' \
                    #           f'-p {entry[2]} -m {entry[1]} -l {lib_path} -o out -log log_{lib[0]}.log -np {args.np}' \
                    #           f'-v {args.voxel}'
                    #
                    # cu.memory_supervised_lsf_run(command, lib[0], out_path, start_mem=args.mem, np=args.np)

                    command = 'bsub -o o_{} -e e_{} -M {} -n {} strudel_mapMotifValidation.py ' \
                              '-p {} -m {} -l {} -o out -log log_{}.log -np {} {}'.format(lib[0], lib[0], args.mem, args.np, entry[2], entry[1], lib_path, lib[0], args.np, segm)
                    # print(f'running {command}')

                    subprocess.call(command, cwd=out_path, shell=True)


if __name__ == '__main__':
    main()


import pytest
import numpy as np
from threed_strudel.utils import bio_utils
from threed_strudel.parse.map_parser import MapParser
from threed_strudel.chop.chop_map import ChopMap
from threed_strudel.validation.map_motif_validation import ComputeScores
import threed_strudel.validation.map_motif_validation as valid


# def test_check_scores_completeness():
#
#     name_list = ['glu-262-A', 'phe-263-A', 'leu-264-A', 'leu-265-A', 'asn-266-A']
#     name_list_plus1 = ['glu-262-A', 'phe-263-A', 'leu-264-A', 'leu-265-A', 'asn-266-A', 'asn-267-A']
#     scores_path_incomplete = 'data/in/validate/scores_top_incomplete.csv'
#     scores_path = 'data/in/validate/scores_top.csv'
#
#     incomplete1 = ComputeScores.check_scores_completeness(name_list_plus1, scores_path, 'some_log')
#     incomplete2 = ComputeScores.check_scores_completeness(name_list, scores_path_incomplete, 'some_log')
#
#     complete = ComputeScores.check_scores_completeness(name_list, scores_path, 'some_log')
#
#     assert not incomplete1
#     assert not incomplete2
#     assert complete

def test_csv_to_top_cav_scores_only():
    incomplete_in = 'data/in/validate/incomplete_scores.csv'
    incomplete_out = 'data/in/validate/incomplete_scores_out_web.csv'
    valid.csv_to_top_csv_scores_only(incomplete_in, incomplete_out)



import os
import csv
import sys
from multiprocessing import Process, Manager, Lock
from datetime import datetime
import multiprocessing
from shutil import copy, rmtree
import json
import argparse
import operator
import psutil
# from chimera import runCommand as rc
# from chimera import openModels as om, selection
# from FitMap.fitcmd import fitmap
from chimerax.core.commands import Command
from chimerax.map.volume import Volume
from chimerax.atomic.structure import AtomicStructure
import logging


log = logging.getLogger(__name__)
rc = Command(session)
SD_LEVEL = 3


def run_x(command):
    try:
        return rc.run(command)[0][0]
    except TypeError:
        return None


def open_model(path):
    try:
        model_obj = session.open_command.open_data(path)[0][0]
        if type(model_obj) == AtomicStructure or type(model_obj) == Volume:
            session.models.add([model_obj])
            return model_obj
    except AttributeError:
        model_obj = run_x(f'open {path}')
        if type(model_obj) == AtomicStructure or type(model_obj) == Volume:
            return model_obj
        else:
            log.error(f'Could not get model object reference after opening:\n {path}')


def read_motif_lib(motif_lib_dir):
    """
    Reads motif library
    :param motif_lib_dir: library path
    :return: list of map model pair paths
    """
    lib_pairs = []
    files = os.listdir(motif_lib_dir)
    pdb_names = [i for i in files if i.endswith('.cif') and not i.startswith('.')]
    for name in pdb_names:
        prefix = name.split('.cif')[0]
        map_path = os.path.join(motif_lib_dir, prefix + '.map')
        mrc_path = os.path.join(motif_lib_dir, prefix + '.mrc')
        if os.path.exists(map_path):
            lib_pairs.append([prefix, os.path.join(motif_lib_dir, name), map_path])
        elif os.path.exists(mrc_path):
            lib_pairs.append([prefix, os.path.join(motif_lib_dir, name), mrc_path])
    return lib_pairs

def load_lib(lib):
    log.debug('Loading motif library')
    l_lib = []
    for mot in lib:
        # mod = run_x('open ' + mot[1])
        # vol = run_x('open ' + mot[2])
        mod = open_model(mot[1])
        vol = open_model(mot[2])
        l_lib.append([mot[0], mod, vol])
    return l_lib


def main():
    st_arg = 1
    for item in sys.argv:
        if str(item).startswith('--'):
            st_arg += 1
    sys.argv = sys.argv[st_arg:]

    parser = argparse.ArgumentParser(description='Score map')
    parser.add_argument("-sl", "--strudel_lib", dest="lib", required=True, help="Motif library in_dir")
    parser.add_argument("-l", "--log", dest="log", required=True, help="Log file")
    parser.add_argument("-o", "--out", dest="csv", required=True, help="csv_path")

    args = parser.parse_args()

    logging.basicConfig(filename=args.log, level=logging.INFO, format='%(levelname)s:  %(message)s')

    data = []

    lib_pairs = read_motif_lib(args.lib)
    loaded_lib = load_lib(lib_pairs)

    for item in loaded_lib:
        row = {}
        row['target'] = item[0]
        t_mod = item[1]
        t_map = item[2]
        t_mod_sel = '#{}@c,ca,n'.format(t_mod.id_string)

        for mot in loaded_lib:
            if mot[0] == item[0]:
                row[mot[0]] = 1
            else:
                l_mod = mot[1]
                l_map = mot[2]

                l_mod_sel = '#{}@c,ca,n'.format(l_mod.id_string)

                # run_x(f'align {l_mod_sel} to {t_mod_sel}')

                # run_x(f'view position #{l_map.id_string} sameAsModels #{l_mod.id_string}')
                run_x(f'volume #{l_map.id_string} sdLevel {SD_LEVEL}')
                run_x(f'volume #{t_map.id_string} sdLevel {SD_LEVEL}')

                fit1 = run_x(f'fitmap #{l_map.id_string} inMap #{t_map.id_string} metric correlation')
                correlation1 = fit1.correlation()

                fit2 = run_x(f'fitmap #{t_map.id_string} inMap #{l_map.id_string} metric correlation')
                correlation2 = fit2.correlation()

                min_corr = round(min([correlation1, correlation2]), 5)
                if min_corr < 0:
                    log.info(f'cor: {min_corr}, {item[0]}, {mot[0]}')
                row[mot[0]] = min_corr
                run_x('view orient; view initial')

        data.append(row)

    with open(args.csv, mode='w') as csv_file:
        fieldnames = [key for key in data[0].keys()]
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
        writer.writeheader()
        for item in data:
            writer.writerow(item)
    run_x('exit')


main()
"""
usage
'ChimeraX_path --nogui  this_script -sl strudel_libpath -l log_path -o csv_path > chimera.log

"""









from __future__ import absolute_import, division, print_function, unicode_literals

from distutils.spawn import find_executable
import os

# ChimeraX executable path
# CHIMERA_PATH = '/nfs/msd/work2/andrei/soft/chimerax/18-12/redhat/opt/UCSF/ChimeraX-daily/bin/ChimeraX'
CHIMERA_PATH = '/Applications/ChimeraX-1.2.5.app/Contents/bin/ChimeraX'
#'/Applications/ChimeraX-1.1.1.app/Contents/MacOS/ChimeraX'


def check_chimerax_executable():
    out = find_executable(CHIMERA_PATH)
    return out


def no_chimerax_warning():
    message = 'WARNING: could not find ChimeraX\n please run:\n'\
              'strudel_setChimeraX.py path_to_ChimeraX_executable'
    return message


def no_chimerax_error():
    message = 'ERROR: could not find ChimeraX\n please run:\n'\
              'strudel_setChimeraX.py path_to_ChimeraX_executable'
    return message


if not check_chimerax_executable():
    print(no_chimerax_warning())


import os
import numpy as np
from scipy.interpolate import RegularGridInterpolator

import threed_strudel.utils.functions as func
import threed_strudel.lib as lib_module


def get_molprobity_file_path(residue):
    head = os.path.dirname(lib_module.__file__)
    file_path = os.path.join(head, 'molprobity', 'rota8000-{}.data'.format(residue.lower()))
    if os.path.exists(file_path):
        return file_path
    else:
        raise Exception("could not find molprobity data file for: " + residue)

def generate_molprobity_interpolator(residue):
    '''
    Load a MolProbity data set and return a SciPy RegularGridInterpolator
    object.
    '''
    data_file = get_molprobity_file_path(residue)
    with open(data_file, 'rt', encoding='utf-8') as infile:
        infile.readline()
        # Get number of dimensions
        ndim = int(infile.readline().split()[-1])
        # Throw away the next line - it's just headers
        infile.readline()
        lower_bound = []
        upper_bound = []
        number_of_bins = []

        step_size = []
        first_step = []
        last_step = []
        axis = []

        # Read in the header to get the dimensions and step size for each
        # axis, and initialise the axis arrays
        for i in range(ndim):
            line = infile.readline().split()
            lb = float(line[2])
            lower_bound.append(lb)
            ub = float(line[3])
            upper_bound.append(ub)
            nb = int(line[4])
            number_of_bins.append(nb)

            ss = (ub - lb) / nb
            step_size.append(ss)
            # Values are at the midpoint of each bin
            fs = lb + ss / 2
            first_step.append(fs)
            ls = ub - ss / 2
            last_step.append(ls)
            axis.append(np.linspace(fs, ls, nb))

        data = np.loadtxt(infile)
    full_grid = np.zeros(number_of_bins)
    # Convert each coordinate to an integral number of steps along each
    # axis
    axes = []
    for i in range(ndim):
        axes.append([])

    for i in range(ndim):
        ss = step_size[i]
        fs = first_step[i]
        lb = lower_bound[i]
        axis_vals = data[:, i]
        axes[i] = (((axis_vals - ss / 2 - lb) / ss).astype(int))

    full_grid[tuple(axes)] = data[:, ndim]

    # Replace all zero or negative values with the minimum positive non-zero
    # value, so that we can use logs
    full_grid[full_grid <= 0] = np.min(full_grid[full_grid > 0])
    # Finally, convert the axes to radians
    axis = np.array(axis)
    # axis = axis / 180 * pi

    return RegularGridInterpolator(axis, full_grid, bounds_error=True)


def read_rotamer_names(bounds_file_path, out_json):

    data = {}
    names = []
    with open(bounds_file_path, 'r') as infile:
        lines = infile.readlines()
        for line in lines:
            s_line = line.split('=')
            r_name, rot_name = s_line[0].split()
            if (r_name, rot_name) not in names:
                names.append((r_name, rot_name))
            if r_name not in data.keys():
                data[r_name] = []
                r_id = 1
            bounds = s_line[-1]
            bounds = bounds.strip().strip("\"")

            bounds = list(map(int, bounds.split(", ")))
            print(bounds)
            r_dict = {'id': r_id, 'name': rot_name}
            for i in range(0, len(bounds), 2):
                print(i)
                delta = bounds[i+1] - bounds[i]
                mid = bounds[i] + delta / 2
                r_dict['chi_{}'.format(int((i+2)/2))] = mid
                r_dict['chi-width_{}'.format(int((i + 2) / 2))] = delta
            data[r_name].append(r_dict)
            r_id += 1
        # print(data)
        # print(len(names))
    func.save_json(out_json, data)




# data_dir = '/Users/andrei/Downloads/reference_data-master/Top8000/Top8000_rotamer_pct_contour_grids/'
# c_dir = '/Users/andrei/Downloads/reference_data-master/Top8000/Top8000_rotamer_pct_contour_grids/'
#
# prefix = 'rota8000-arg'
# start = time.time()
# interp = generate_molprobity_interpolator(data_dir, prefix)
# print('built time', time.time()-start)
# start = time.time()
# # print(interp([180.6/180*pi, 77.8/180*pi, 175.2/180*pi, 143.4/180*pi]))
# print(interp([180.6, 77.8, 175.2, 143.4]))
# print('interp time', time.time()-start)


"""
chop_map.py

A collection of methods which allow to chop a map around an atomic model.
The map can be chopped in three different ways:
- using a cub around the atomic residue with hard edges
- using certain radius around atomic residue with hard edges
- using certain radius around atomic residue with soft edges

Copyright [2013] EMBL - European Bioinformatics Institute
Licensed under the Apache License, Version 2.0 (the
"License"); you may not use this file except in
compliance with the License. You may obtain a copy of
the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied. See the License for the
specific language governing permissions and limitations
under the License.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

__author__ = 'Andrei Istrate'
__email__ = 'andrei@ebi.ac.uk'
__date__ = '2018-05-29'

import os
import math
from random import randint
import numpy as np
from threed_strudel.parse.map_parser import MapParser
import threed_strudel.utils.bio_utils as bu
import psutil
p = psutil.Process()

class ChopMap:
    """
    Class for chopping map around an atomic model
    """

    # def __init__(self, in_model=None, in_map=None):
    #     self.model = in_model
    #     self.map = in_map
    #     self.sett_model_map()
    #
    # def sett_model_map(self):
    #     """
    #     Loads the map and model
    #     """
    #     if self.map is not None:
    #         self.map = MapParser(self.map)
    #     if self.model is not None:
    #         self.model = bio_utils.load_structure(self.model)

    def chop_cube_list(self, in_model_list, in_map, cube_padding, zero_origin=True):
        """
         Chop map using a cubic box around the model
         :param in_model_list: list of biopython model objects
         :param in_map: path to the input map or strudel map object
         :param cube_padding: distance fom any cube edge to the nearest atom
         :param zero_origin: boolean,
         :return: list of map objects, list of translation matrices
         """

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        shifts_list = []
        chopped_map_obj_list = []

        for model in in_model_list:
            out_map, shifts = self.chop_cube(model, in_map, cube_padding, zero_origin)
            chopped_map_obj_list.append(out_map)
            shifts_list.append(shifts)

        return chopped_map_obj_list, shifts_list

    @staticmethod
    def chop_cube(in_model, in_map, cube_padding=5, zero_origin=True, out_map_path=None):
        """
         Chop map using a cubic box around the model
         :param in_model: biopython model object
         :param in_map: path to the input map or strudel map object
         :param cube_padding: distance fom any cube edge to the nearest atom
         :param out_map_path: output map path
         :param zero_origin: boolean,
         :return: map object, translation matrix
         """

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        if isinstance(in_model, str):
            in_model = bu.load_structure(in_model)

        atoms_coord = []
        for atom in in_model.get_atoms():
            atoms_coord.append(atom.coord)

        delta = round(cube_padding / in_map.voxel_size[0])
        # Get the indices in the map grid that correspond to the atom coordinates
        x_indices = []
        y_indices = []
        z_indices = []
        for atom in atoms_coord:
            x_index, y_index, z_index = in_map.coord_to_index(atom)

            x_indices.append(x_index)
            y_indices.append(y_index)
            z_indices.append(z_index)
        # Find the voxel located in the middle of the atomic residue
        # and the maximum molecule size in grid points
        deltas = []
        minim = min(x_indices)
        dx = max(x_indices) - minim
        middle_x = int(round(dx / 2 + minim))
        deltas.append(dx)
        minim = min(y_indices)
        dy = max(y_indices) - minim
        middle_y = int(round(dy / 2 + minim))
        deltas.append(dy)
        minim = min(z_indices)
        dz = max(z_indices) - minim
        middle_z = int(round(dz / 2 + minim))
        deltas.append(dz)

        max_d = max(deltas)
        # Calculate the size of the cube
        radius = int(round(max_d / 2 + delta))
        new_dimension = radius * 2

        # Ensure that the new grid size has no prime numbers greater than 19
        # new_dimension = func.find_good_grid([new_dimension, new_dimension, new_dimension])[0]
        # print(new_dimension)
        # hard_radius = int(new_dimension / 2)

        # Create a numpy array to store the chopped voxels
        new_data = np.zeros((new_dimension, new_dimension, new_dimension), dtype='float32')

        # Assign voxel values
        for x in range(new_dimension):
            for y in range(new_dimension):
                for z in range(new_dimension):
                    x1 = x + middle_x - radius
                    y1 = y + middle_y - radius
                    z1 = z + middle_z - radius
                    if any([x1 < 0, y1 < 0, z1 < 0]):
                        pass
                    else:
                        try:
                            new_data[x, y, z] = in_map.data[x1, y1, z1]
                        except IndexError:
                            pass
        # Calculate the new cell size
        voxel_size = in_map.voxel_size
        new_cell = (round(new_dimension * voxel_size[0], 3),
                    round(new_dimension * voxel_size[1], 3),
                    round(new_dimension * voxel_size[2], 3))

        # Calculate the shifts applied to the chopped map
        if not zero_origin:
            shifts = np.array([0, 0, 0])
        else:
            index_shifts = in_map.index_shifts
            x_shift = (middle_z - radius) * voxel_size[0] + index_shifts[0] * voxel_size[0]
            y_shift = (middle_y - radius) * voxel_size[1] + index_shifts[1] * voxel_size[1]
            z_shift = (middle_x - radius) * voxel_size[2] + index_shifts[2] * voxel_size[2]

            shifts = np.array([x_shift, y_shift, z_shift])

        new_data[np.isnan(new_data)] = 0

        out_map = MapParser('')
        out_map.data = new_data
        out_map.cell = new_cell
        out_map.cellb = in_map.cellb
        out_map.voxel_size = voxel_size
        n_st = in_map.n_start
        if not zero_origin:
            origin = ((middle_z - radius + n_st[0]) * out_map.voxel_size[0] + in_map.origin[0],
                      (middle_y - radius + n_st[1]) * out_map.voxel_size[1] + in_map.origin[1],
                      (middle_x - radius + n_st[2]) * out_map.voxel_size[2] + in_map.origin[2])

            out_map.set_origin(origin)
            out_map.n_start = (0, 0, 0)

        if out_map_path is not None:
            out_map.write_map(out_map_path)

        return out_map, shifts

    @staticmethod
    def check_cut_vals(model, in_map, out_map, shifts):
        """
        Checks the map values at 5 atoms positions
        :param model:
        :param in_map:
        :param out_map:
        :param shifts:
        :return:
        """
        atoms = [a for a in model.get_atoms()]
        in_coords = []
        out_coords = []

        for i in range(5):
            n = randint(0, len(atoms))
            try:
                in_coords.append(atoms[n].coord)

            except IndexError:
                pass
        for coord in in_coords:
            tmp_coord = list(coord)
            for i in range(3):
                tmp_coord[i] = tmp_coord[i] - shifts[i]
            out_coords.append(tmp_coord)

        in_vals = []
        out_vals = []

        for i in range(len(in_coords)):
            in_indices = in_map.coord_to_index_int(in_coords[i])
            out_indices = out_map.coord_to_index_int(out_coords[i])
            neg_indices = sum(n < 0 for n in in_indices + out_indices)
            if neg_indices > 0:
                in_vals.append(None)
                out_vals.append(None)
            else:
                try:
                    in_vals.append(round(in_map.data[in_indices], 5))
                except IndexError:
                    in_vals.append(None)
                try:
                    out_vals.append(round(out_map.data[out_indices], 5))
                except IndexError:
                    out_vals.append(None)

        if in_vals == out_vals and not all([v is None for v in in_vals]):
            return True
        else:
            print('IN', in_vals)
            print('OU', out_vals)
            return False

    @staticmethod
    def chop_soft_radius(model, in_map, out_map=None, hard_radius=3, soft_radius=2, mask_path=None):
        """
        Chop map using a soft mask with a given radius (hard_radius + soft_radius)
        around the atomic residue. A cosine function is used to create the soft mask.
        :param mask_path: mask output path
        :param model: biopython model object or atomic model path
        :param in_map: path to the input map or strudel map object
        :param out_map: out_map: output map path
        :param hard_radius: hard radius
        :param soft_radius: soft radius, cosine function
        :return strudel map object if out_map not given otherwise None
         """
        if isinstance(model, str):
            model = bu.load_structure(model)
        # Get atom coordinates
        atoms_coord = []
        for atom in model.get_atoms():
            atoms_coord.append(atom.coord)

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        voxel_size = in_map.voxel_size
        aver_voxel_size = sum(voxel_size) / 3
        delta1 = hard_radius
        delta2 = hard_radius + soft_radius

        # Create a numpy array for mask
        shape = in_map.data.shape
        mask = np.zeros(shape, dtype='float32')
        for coord in atoms_coord:
            x_index, y_index, z_index = in_map.coord_to_index_int(coord)

            rx = int(round(delta2 / voxel_size[0]))
            ry = int(round(delta2 / voxel_size[1]))
            rz = int(round(delta2 / voxel_size[2]))

            for x in range(x_index - rx, x_index + rx):
                for y in range(y_index - ry, y_index + ry):
                    for z in range(z_index - rz, z_index + rz):
                        # Calculate the distance between the current atom and the current voxel
                        if any([x < 0, y < 0, z < 0]):
                            continue
                        d = aver_voxel_size * math.sqrt((x - x_index) ** 2 + (y - y_index) ** 2 + (z - z_index) ** 2)

                        # Assign mask values based to the distance to the atoms
                        if d <= delta1:
                            try:
                                mask[x, y, z] = 1
                            except IndexError:
                                pass
                        elif delta1 < d < delta2:
                            try:
                                mask[x, y, z] += (math.cos((math.pi / soft_radius) * (d - delta1)) + 1) / 2
                                # if intensity value became > 1 it is set to 1
                                if mask[x, y, z] > 1:
                                    mask[x, y, z] = 1
                            except IndexError:
                                pass

        # Apply the mask to the map data
        in_map.data = in_map.data * 1000

        final = (mask * (in_map.data))
        # print('final', final[3,3,3])
        # print(final[1][1][:])
        # final[final < 0] = 0
        # print(final[1][1][:])
        # import sys
        # np.set_printoptions(threshold=sys.maxsize)
        # print('out\n\n\n')
        # # print(in_map.data)
        # print('Inp max', np.max(in_map.data))
        # # print(final)
        # print('out max', np.max(final))
        out_map_obj = MapParser('')
        out_map_obj.copy_header(in_map)
        out_map_obj.data = final

        if mask_path is not None:
            mask_ob = MapParser('')
            mask_ob.copy_header(in_map)
            mask_ob.data = mask
            mask_ob.write_map(mask_path)

        if out_map is not None:
            out_map_obj.write_map(out_map)

        return out_map_obj

    @staticmethod
    def find_near_atoms(atom, model, distance=6):
        """
        Finds all atoms which are closer than "distance" from the target atom
        :param atom: target atom (center of search)
        :param model: biopython structure object
        :param distance: search radius
        :return: list of biopython atom objects
        """
        close = []
        # print("Atom_parent", atom.parent.id[1], type(atom.parent.id[1]))
        for atom1 in model.get_atoms():
            if atom1.parent.id != atom.parent.id or atom1.parent.parent.id != atom.parent.parent.id:
                d = atom1 - atom
                if d < distance:
                    close.append(atom1)
        close = list(set(close))
        # Filter out flanking residues BB atoms
        # close = [a for a in close if a.parent.id[1] not in [atom.parent.id[1]-1, atom.parent.id[1]+1]]
        filtered = []
        for a in close:
            if a.get_name() not in ['N', 'C', 'O', 'CA']:
                filtered.append(a)
            elif a.parent.id[1] not in [atom.parent.id[1] - 1, atom.parent.id[1] + 1]:
                filtered.append(a)
        return filtered


    @staticmethod
    def print_mem():
        mem = psutil.Process(p.pid).memory_info()
        return mem.rss / 1000000

    def chop_soft_radius_watershed_slow(self, model, in_map, whole_model, shifts=None, out_map=None,
                                       radius=2, soft_radius=1, mask_path=None):
        """
        TODO: requires more testing
        Chop map using a soft mask with a given radius (hard_radius + soft_radius) around an amino acid residue residue.
        A cosine function is used to create the soft mask. Similar to chop_soft_radius but avoids
        cutting neighboring residues side chains map. To do so, it finds the closest atom
        (which does not belong to the guide model) for each atom in the guide model and
        tries to separate the map between them.
        It can be used to chop map around bigger models but may take long for big objects.
        :param whole_model: biopython model object. The complete model of which the guide model is a part of
        :param model: biopython atomic residue object
        :param in_map: in_dir to the input map
        :param out_map: out_map: in_dir for the chopped map
        :param radius: hard radius
        :param soft_radius: soft radius (a cosine function is applied for it)
        :return out_map_obj: map object
         """
        # Get atom coordinates
        if shifts is None:
            shifts = np.array([0, 0, 0])

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        voxel_size = in_map.voxel_size
        aver_voxel_size = sum(voxel_size) / 3

        # Create a numpy array for mask
        shape = in_map.data.shape
        mask = np.zeros(shape, dtype='float32')

        r = int(round((radius + soft_radius) / aver_voxel_size))
        for atom in model.get_atoms():
            xyz = in_map.coord_to_index(atom.coord - shifts)
            xyz_int = in_map.coord_to_index_int(atom.coord - shifts)

            near_atoms = []
            if atom.get_name() not in ['C', 'CA']:
                near_atoms = self.find_near_atoms(atom, whole_model, distance=(radius + soft_radius) * 2)

            for x in range(xyz_int[0] - r, xyz_int[0] + r):
                for y in range(xyz_int[1] - r, xyz_int[1] + r):
                    for z in range(xyz_int[2] - r, xyz_int[2] + r):
                        if any([x < 0, y < 0, z < 0]):
                            continue
                        near_ds = [100]
                        for n_atom in near_atoms:
                            n_xyz = in_map.coord_to_index(n_atom.coord - shifts)
                            dn = aver_voxel_size * math.sqrt((x - n_xyz[0]) ** 2 + (y - n_xyz[1]) ** 2
                                                             + (z - n_xyz[2]) ** 2)
                            near_ds.append(dn)
                        dn = min(near_ds)

                        # Calculate the distance between the current atom and the current voxel
                        d = aver_voxel_size * math.sqrt((x - xyz[0]) ** 2 + (y - xyz[1]) ** 2 + (z - xyz[2]) ** 2)
                        if d > dn * 1.3:
                            continue
                        elif dn < radius + soft_radius:
                            delta2 = min((d + dn) * 0.65, radius + soft_radius)
                            delta1 = delta2 - soft_radius
                        else:
                            delta2 = radius + soft_radius
                            delta1 = radius
                        # Assign mask values based to the distance to the atoms
                        if d < delta1:
                            try:
                                mask[x, y, z] = 1
                            except IndexError:
                                pass
                        elif delta1 < d < delta2:
                            try:
                                mask[x, y, z] += (math.cos((math.pi / soft_radius) * (d - delta1)) + 1) / 2
                            except IndexError:
                                pass
        mask[mask > 1] = 1
        final = (mask * in_map.data)
        out_map_obj = MapParser('')
        out_map_obj.copy_header(in_map)
        out_map_obj.data = final

        if mask_path is not None:
            mask_ob = MapParser('')
            mask_ob.copy_header(in_map)
            mask_ob.data = mask
            mask_ob.write_map(mask_path)

        if out_map is not None:
            out_map_obj.write_map(out_map)
        return out_map_obj

    def chop_soft_radius_watershed(self, model, in_map, whole_model, shifts=None, out_map=None,
                                   radius=2, soft_radius=1, asymmetric_delta=0.5, mask_path=None):
        """
        Chop map using a soft mask with a given radius (hard_radius + soft_radius) around an amino acid residue residue.
        A cosine function is used to create the soft mask. Similar to chop_soft_radius but avoids
        cutting neighboring residues side chains map. To do so, it creates two masks: a soft edge mask (var: mask)
        around the guide model and another soft edge mask (var: outer_mask) around the atoms which are near the guide
        model atoms (d < hard_radius + soft_radius). The final mask is given by: mask = (mask - outer_mask * mask).
        It can be used to chop map around bigger models but may take long for big objects.
        :param whole_model: biopython model object. The complete model of which the guide model is a part of
        :param shifts: between model and map
        :param mask_path: mask output path
        :param model: biopython atomic residue object
        :param in_map: in_dir to the input map
        :param out_map: out_map: in_dir for the chopped map
        :param radius: hard radius
        :param soft_radius: soft radius
        :param asymmetric_delta:
         """
        # r1 - hard radius for near atoms
        r1 = radius - asymmetric_delta
        # r2 - soft radius for near atoms
        r2 = r1 + soft_radius

        # Get atom coordinates
        if shifts is None:
            shifts = np.array([0, 0, 0])

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        voxel_size = in_map.voxel_size
        aver_voxel_size = sum(voxel_size) / 3
        delta1 = radius
        delta2 = radius + soft_radius

        # Create a numpy array for mask
        shape = in_map.data.shape
        mask = np.zeros(shape, dtype='float32')
        outer_mask = np.zeros(shape, dtype='float32')

        r = int(round((radius + soft_radius) / aver_voxel_size))
        near_atoms = []
        import time
        near_time = 0
        for atom in model.get_atoms():
            xyz = in_map.coord_to_index(atom.coord - shifts)
            xyz_int = in_map.coord_to_index_int(atom.coord - shifts)

            t = time.time()
            if atom.get_name() not in ['C', 'CA', 'N', 'O']:
                # near_atoms += self.find_near_atoms(atom, whole_model, distance=(radius + soft_radius) * 2)
                near_atoms += self.find_near_atoms(atom, whole_model, distance=4)
            near_time += time.time() - t
            for x in range(xyz_int[0] - r, xyz_int[0] + r):
                for y in range(xyz_int[1] - r, xyz_int[1] + r):
                    for z in range(xyz_int[2] - r, xyz_int[2] + r):
                        if any([x < 0, y < 0, z < 0]):
                            continue
                        # Calculate the distance between the current atom and the current voxel
                        d = aver_voxel_size * math.sqrt((x - xyz[0]) ** 2 + (y - xyz[1]) ** 2 + (z - xyz[2]) ** 2)
                        if d <= delta1:
                            try:
                                mask[x, y, z] = 1
                            except IndexError:
                                pass
                        elif delta1 < d < delta2:
                            try:
                                mask[x, y, z] += (math.cos((math.pi / soft_radius) * (d - delta1)) + 1) / 2
                                # if intensity value became > 1 it is set to 1
                                if mask[x, y, z] > 1:
                                    mask[x, y, z] = 1
                            except IndexError:
                                pass
        mask[mask > 1] = 1
        near_atoms = list(set(near_atoms))
        # print('NEAR', len(near_atoms), near_atoms)
        # print("NEAR time ", near_time)
        for atom in near_atoms:
            xyz = in_map.coord_to_index(atom.coord - shifts)
            xyz_int = in_map.coord_to_index_int(atom.coord - shifts)

            for x in range(xyz_int[0] - r, xyz_int[0] + r):
                for y in range(xyz_int[1] - r, xyz_int[1] + r):
                    for z in range(xyz_int[2] - r, xyz_int[2] + r):

                        # Calculate the distance between the current atom and the current voxel
                        d = aver_voxel_size * math.sqrt((x - xyz[0]) ** 2 + (y - xyz[1]) ** 2 + (z - xyz[2]) ** 2)
                        if d <= r1:
                            try:
                                outer_mask[x, y, z] = 1
                            except IndexError:
                                pass
                        elif r1 < d < r2:
                            try:
                                outer_mask[x, y, z] += (math.cos((math.pi / soft_radius) * (d - r1)) + 1) / 2
                                # if intensity value became > 1 it is set to 1
                                if outer_mask[x, y, z] > 1:
                                    outer_mask[x, y, z] = 1
                            except IndexError:
                                pass
        outer_mask[outer_mask > 1] = 1
        outer_mask[mask == 0] = 0
        mask = (mask - outer_mask * mask)
        final = (mask * in_map.data)

        out_map_obj = MapParser('')
        out_map_obj.copy_header(in_map)
        out_map_obj.data = final

        if mask_path is not None:
            mask_ob = MapParser('')
            mask_ob.copy_header(in_map)
            mask_ob.data = mask
            mask_ob.write_map(mask_path)

        # mask_ob = MapParser('')
        # mask_ob.copy_header(in_map)
        # mask_ob.data = mask
        #
        # outer_mask_ob = MapParser('')
        # outer_mask_ob.copy_header(in_map)
        # outer_mask_ob.data = outer_mask

        if out_map is not None:
            out_map_obj.write_map(out_map)
        return out_map_obj  # , mask_ob, outer_mask_ob

    @staticmethod
    def chop_hard_radius(model, in_map, out_map, radius=3, mask_path=None):
        """
        Chop map using a hard mask with a given hard_radius around the atomic residue.
        :param model: biopython atomic residue object
        :param in_map: in_dir to the input map
        :param out_map: out_map: in_dir for the chopped map
        :param radius: mask radius around atoms
        :param mask_path: mask output path
        :return out_map_obj: masked map object
        """
        # Get atom coordinates
        atoms_coord = []
        for atom in model.get_atoms():
            atoms_coord.append(atom.coord)

        if isinstance(in_map, MapParser):
            pass
        elif os.path.exists(in_map):
            in_map = MapParser(in_map)
        else:
            raise Exception('in_map should be MapParser object or a map file path not {}'.format(type(in_map)))

        voxel_size = in_map.voxel_size
        aver_voxel_size = sum(voxel_size) / 3
        shape = in_map.data.shape
        mask = np.zeros(shape, dtype='float32')

        for coord in atoms_coord:
            x_index, y_index, z_index = in_map.coord_to_index_int(coord)

            rx = int(round(radius / voxel_size[0]))
            ry = int(round(radius / voxel_size[0]))
            rz = int(round(radius / voxel_size[0]))
            for x in range(x_index - rx, x_index + rx):
                for y in range(y_index - ry, y_index + ry):
                    for z in range(z_index - rz, z_index + rz):
                        # Calculate the distance between the current atom and the current voxel
                        d = aver_voxel_size * math.sqrt((x - x_index) ** 2 + (y - y_index) ** 2
                                                        + (z - z_index) ** 2)
                        # Assign mask values based to the distance to the atoms
                        if d < radius:
                            mask[x, y, z] = 1

        # Apply the mask to the map data
        final = (mask * in_map.data)
        # Save the chopped map

        if mask_path is not None:
            mask_ob = MapParser('')
            mask_ob.copy_header(in_map)
            mask_ob.data = mask
            mask_ob.write_map(mask_path)

        out_map_obj = MapParser('')
        out_map_obj.copy_header(in_map)
        out_map_obj.data = final

        if out_map is not None:
            out_map_obj.write_map(out_map)

        return out_map_obj



#!/Users/andrei/PycharmProjects/for_testing/venv/bin/python

import threed_strudel
import os
import sys
from distutils.spawn import find_executable

config_path = os.path.abspath(os.path.join(os.path.dirname(threed_strudel.__file__), 'configure.py'))

CHIMERA_PATH = sys.argv[1]
tmp = CHIMERA_PATH + '.app'

if os.path.exists(tmp):
    CHIMERA_PATH = tmp

out = find_executable(CHIMERA_PATH)

if out is None:
    for subdir in ['Contents/MacOS/ChimeraX', 'bin/ChimeraX']:
        tmp = os.path.join(CHIMERA_PATH, subdir)
        if os.path.exists(tmp):
            out = find_executable(tmp)
            if out is not None:
                CHIMERA_PATH = tmp
                break

if out is None:
    print('ERROR: \n{}\n is not an executable'.format(CHIMERA_PATH))

else:
    with open(config_path, 'r') as f:
        lines = f.readlines()
        for i, line in enumerate(lines):
            words = line.split()
            if len(words) > 1:
                if words[0] == 'CHIMERA_PATH':
                    words[-1] = '"{}"'.format(CHIMERA_PATH)
                    lines[i] = ' '.join(words) + '\n'
    with open(config_path, 'w') as f:
        f.writelines(lines)